#include <cstdio>
#include "httpfun.hh"

const std::string htmlencode(const std::string& s)
{
    std::string result;
    for(std::string::const_iterator i = s.begin(); i != s.end(); ++i)
    {
        if(*i == '&')
            result += "&amp;";
        else if(*i == '<')
            result += "&lt;";
        else if(*i == '"')
            result += "&quot;";
        else result += *i;
    }
    return result;
}

const std::string urlencode(const std::string& s)
{
    std::string result;
    for(std::string::const_iterator i = s.begin(); i != s.end(); ++i)
    {
        if((*i >= 48 && *i <= 57)
        || (*i >= 65 && *i <= 90)
        || (*i >= 97 && *i <= 122)
        || (*i == 46))
            result += *i;
        else
        {
            char Buf[4];
            std::sprintf(Buf, "%02X", *i);
            result += Buf;
        }
    }
    return result;
}

const std::string urldecode(const std::string& input)
{
    std::string result;
    for(unsigned a=0; a<input.size(); ++a)
        if(input[a] == '+')
            result += ' ';
        else if(input[a] == '%')
        {
            unsigned c = 0;
            std::sscanf(input.c_str()+a+1, "%2X", &c);
            result += (char)c;
            a += 2;
        }
        else result += input[a];
    return result;
}

void SplitDecodeQuery(std::string& request,
                      std::multimap<std::string, std::string>& queryparams)
{
    //fprintf(stderr, "Request(%s)\n", request.c_str());
    
    std::string::size_type p = request.find('?');
    if(p != request.npos)
    {
        std::string query = request.substr(p+1);
        request.erase(p);
        
        //fprintf(stderr, "Query(%s)\n", query.c_str());
        
        for(p = 0; p < query.size(); )
        {
            std::string::size_type nextp = query.find('&', p);
            if(nextp == query.npos) nextp = query.size();
            
            std::string token = query.substr(p, nextp-p);
            
            //fprintf(stderr, "  Token(%s)\n", token.c_str());
            
            std::string::size_type eqpos = token.find('='), eqaftpos = eqpos+1;
            if(eqpos == token.npos) eqpos = token.size(), eqaftpos = 0;
            
            std::string key   = urldecode(token.substr(0, eqpos));
            std::string value = urldecode(token.substr(eqaftpos));
            
            //fprintf(stderr, "    Key(%s)\n", key.c_str());
            //fprintf(stderr, "    Value(%s)\n", value.c_str());
            queryparams.insert(std::make_pair(key, value));
            
            p = nextp+1;
        }
    }
    
    // finally, unescape request
    request = urldecode(request);
}
